{{ dwd_nRF24L01Alpha01
  Based on nFR24L01 by Nikita Kareev
  Not much of the original object is left.
  I mostly relied on C code posted by Leon
  on the Propeller Forum.
  Version: Alpha01
  By: Duane Degn
  April 9, 2011
  I'm using a modified version of SPI_Asm.
  I removed the delays in the original
  object since these modules can use a
  10MHz SPI clock and require only a 2ns
  delay on the clock after the data pin
  is set.
}}
''****************************************
''*  nFR24L01                            *
''*  Authors: Nikita Kareev              *
''*  See end of file for terms of use.   *
''****************************************
''
'' Nordic nRF24L01 driver
''
'' http://www.sparkfun.com/commerce/product_info.php?products_id=691
''
'' Updated... 6 SEP 2009
''
'' Rev 0.1 
CON
  ' DWD These constants where in the original object.
  ' DWD Not many are used.

  
  'nRF24L01 constants:
  
  'Commands
  R_REGISTER = %0000_0000 '+ reg
  W_REGISTER = %0010_0000 '+ reg
  R_RX_PAYLOAD = %0110_0001         '97
  W_TX_PAYLOAD = %1010_0000         '160
  FLUSH_TX = %1110_0001             '225
  FLUSH_RX = %1110_0010             '226
  REUSE_TX_PL = %1110_0011
  R_RX_PL_WID = %0110_0000           '96
  W_ACK_PAYLOAD = %1010_1000 '+ pipe  168
  W_TX_PAYLOAD_NOACK = %1011_0000    '176
  NOOP = %1111_1111

  'Registers
  CONFIG = $00   '%0000_1000 / $08
  EN_AA = $01  '%0011_1111  / $3F
  EN_RXADDR = $02  '%0000_0011
  SETUP_AW = $03   '%0000_0011
  SETUP_RETR = $04  '%0000_0011
  RF_CH = $05   '%0000_0010
  RF_SETUP = $06  '%0000_1110
  STATUS = $07     '%0000_1110 / $E 
  OBSERVE_TX = $08  '%0000_0000
  RPD = $09   '%0000_0000 
  RX_ADDR_P0 = $0A '$E7E7E7E7E7
  RX_ADDR_P1 = $0B '$C2C2C2C2C2
  RX_ADDR_P2 = $0C  '$C3
  RX_ADDR_P3 = $0D  '$C4 
  RX_ADDR_P4 = $0E  '$C5 
  RX_ADDR_P5 = $0F  '$C6 
  TX_ADDR = $10    '$E7E7E7E7E7
  RX_PW_P0 = $11   '%0000_0000
  RX_PW_P1 = $12   '%0000_0000 
  RX_PW_P2 = $13   '%0000_0000 
  RX_PW_P3 = $14  '%0000_0000 
  RX_PW_P4 = $15  '%0000_0000 
  RX_PW_P5 = $16  '%0000_0000 
  FIFO_STATUS = $17 '%0001_0001 
  DYNPD = $1C   '%0000_0000
  FEATURE = $1D  '%0000_0000

OBJ 

  Spi : "SPI_AsmFast"

VAR
  
  byte SpiSck
  byte SpiMiso
  byte SpiMosi 
  byte SpiCsn    
  byte SpiCe
  byte SpiIrq
  
  byte payloadSize
  'byte mode   

PUB Init(sck, miso, mosi, csn, ce, mode, _payloadSize) 
'' _payloadSize should be between 1 and 32 (inclusive))
'' _mode can be 0 for rx or 1 for tx

  result := Spi.Start(0) - 1
  
  SpiSck := sck
  SpiMiso := miso
  SpiMosi := mosi 
  SpiCsn := csn     
  SpiCe := ce
  
  outa[SpiSck]~
  outa[SpiMosi]~
  outa[SpiCsn]~~                          ' chip select not, active low
  outa[SpiCe]~
  
  dira[SpiSck]~~
  dira[SpiMosi]~~
  dira[SpiCsn]~~
  dira[SpiCe]~~

  payloadSize := _payloadSize
  'mode := _mode
  
  'Configure for receive
  ConfigureRX(mode)             ' I'll need a way to change the mode on the fly.

PUB ConfigureRX(mode)

{{
  Configure nRF24L01 registers for receive mode.
  1. Data pipe 0 used
  2. RX Address is E7E7E7E7E7 
  3. Data rate is 250kbps - no ESB
  4. No Auto Ack                 
}}
  
  outa[SpiCe]~     

  outa[SpiCsn]~    'max_rt is masked both times
  SpiWrite($20)
  if mode == 0
    SpiWrite($33)
  elseif mode == 1 
    SpiWrite($52) 
  outa[SpiCsn]~~ 

  outa[SpiCsn]~
  SpiWrite($21)
  SpiWrite($00)        'disable autoAck
  outa[SpiCsn]~~

  outa[SpiCsn]~
  SpiWrite($22)
  SpiWrite($01)        'enable data pipe 0
  outa[SpiCsn]~~      
  
  outa[SpiCsn]~   
  SpiWrite($23)         'address width = 5 bytes
  SpiWrite($03)    
  outa[SpiCsn]~~
     
  if mode == 1
    outa[SpiCsn]~
    SpiWrite($24)        'disable Retransmit
    SpiWrite($00)     
    outa[SpiCsn]~~        
    
  outa[SpiCsn]~
  SpiWrite($25)       'setup channel
  SpiWrite($02)       'channel 2              
  outa[SpiCsn]~~     
 
  outa[SpiCsn]~   
  SpiWrite($26)          'write to register 6
  SpiWrite($26)          '250kbps 'high power            
  outa[SpiCsn]~~
 
  outa[SpiCsn]~  
  SpiWrite($31)         
  SpiWrite(payloadSize) ' I'm using a 1 byte payload  
  outa[SpiCsn]~~   

  if mode == 0
    outa[SpiCsn]~
    SpiWrite(RX_ADDR_P0)       'tx addr   
    repeat 5
      SpiWrite($E7)     'write pipe address to tx fifo
    outa[SpiCsn]~~

  if mode == 1
    outa[SpiCsn]~
    SpiWrite(TX_ADDR)       'tx addr   
    repeat 5
      SpiWrite($E7)     'write pipe address to tx fifo
    outa[SpiCsn]~~  
  
 { outa[SpiCsn]~
  SpiWrite($30)       'tx addr   
  repeat 5
    SpiWrite($E7)     'write pipe address to tx fifo
  outa[SpiCsn]~~  }

  outa[SpiCe]~~

PUB ReadPayload(stringToFillPtr) | localIndex
'' I'm not sure if I have to worry which mode the module is in.

{{
  Reads payload from nRF24L01.
  Also flushes RX FIFO and resets IRQ state
}}    

  'Stop receiving
  outa[SpiCe]~
    
  'Read RX payload   
  outa[SpiCsn]~    
  SpiWrite(R_RX_PAYLOAD)       
  repeat payloadSize 
    byte[stringToFillPtr++] := SpiRead  '====================
  outa[SpiCsn]~~
    
  'Flush RX FIFO    
  outa[SpiCsn]~    
  SpiWrite($E2)    
  outa[SpiCsn]~~

  'Reset IRQ 
  outa[SpiCsn]~
  SpiWrite($27)   'clear rx irq
  SpiWrite($40)    
  outa[SpiCsn]~~

  'Start receiving
  outa[SpiCe]~~

PUB SpiRead

  result := Spi.ShiftIn(SpiMiso, SpiSck, Spi#MSBPRE, 8)
   
PUB SpiWrite(byteOut)

  Spi.ShiftOut(SpiMosi, SpiSck, Spi#MSBFIRST, 8, byteOut)

PUB TransmitString(localPtr)  | idx
'' For now this method transmit the a full payload of bytes starting with
'' the memorey location "localPtr".  I should have some global variable
'' that keeps track of which state (rx or tx) the module is in.
'' Right now this object doesn't keep track of the module's state.

  outa[SpiCsn]~

  outa[SpiCsn]~
  SpiWrite($27)    'write status register
  SpiWrite($20)    'clear tx irq       
  outa[SpiCsn]~~   'can be commented out

  outa[SpiCsn]~
  SpiWrite($E1)    '225 flush tx
  outa[SpiCsn]~~

  outa[SpiCsn]~     
  SpiWrite($A0)           '160 write tx payload
  repeat payloadSize
    SpiWrite(localPtr)     
  outa[SpiCsn]~~

  outa[SpiCe]~~
  waitcnt(1000 + cnt)     ' > 10us 800 = 10 us @ 80MHz
  'waitcnt((clkfreq / 1000) * 69 + cnt)  
  outa[SpiCe]~
  